<?php
class SketchApi
{

    protected static $bin = '/usr/bin/perl';
    protected $cf_sketch_api_executable = 'cf-dc-api.pl';
    protected $cf_sketch_api_script_path = '';
    protected $cf_sketch_api_config_file = 'api-config.json';
    public $clonePath = '';
    public $username = '';

    function __construct($data = array())
    {

        if (!empty($data))
        {
            $this->initialize($data);
        }
    }

    function initialize($config = array())
    {


        foreach ($config as $key => $val)
        {
            if (isset($this->$key))
            {
                $method = 'set_' . $key;

                if (method_exists($this, $method))
                {
                    $this->$method($val);
                }
                else
                {
                    $this->$key = $val;
                }
            }
        }
    }

    public static function setBinPath($path)
    {
        self::$bin = $path;
    }

    public static function getBin()
    {

        return self::$bin;
    }

    protected function run_command($command)
    {
        syslog(LOG_DEBUG, "Running DC-API command:: $command");
        $descriptorspec = array(
            1 => array(
                'pipe',
                'w'
            ) ,
            2 => array(
                'pipe',
                'a'
            ) ,
        );
        $pipes = array();
        $resource = proc_open($command, $descriptorspec, $pipes, $this->cf_sketch_api_script_path);

        $stdout = stream_get_contents($pipes[1]);
        $stderr = stream_get_contents($pipes[2]);


        foreach ($pipes as $pipe)
        {
            fclose($pipe);
        }

        $status = trim(proc_close($resource));

        syslog(LOG_DEBUG, "DC-API STDOUT ($status):: $stdout");

        if ($status && !empty($stderr))
        {
            syslog(LOG_DEBUG, "DC-API STDERR ($status):: $stderr");

            throw new Exception($stderr);
        }
        $this->parse_api_response($stdout, $stderr, $command);

        return $stdout;
    }

    private function parse_api_response($response, $stderr, $command = '')
    {
        $data = json_decode($response, true);
        if (!empty($data))
        {
            if (key_exists('api_ok', $data) !== false)
            {
                if ($data['api_ok']['success'] != true)
                {
                    $message = implode(',', $data['api_ok']['errors']);

                    throw new Exception($message);
                }
                else
                {

                    return;
                }
            }
            else if (key_exists('api_error', $data) !== false)
            {
                $message = implode(',', $data['api_error']['errors']);
                $message.= $command;

                throw new Exception($message);
            }
        }

        throw new Exception("undefined exception : $response with command :: " . $command);
    }
    /**
     *
     * @param JSON $command
     * @return type
     */
    public function run($request)
    {
        $json = escapeshellarg($request->generateJson());

        $command = sprintf("%s %s %s %s", self::getBin() , $this->cf_sketch_api_executable, $this->cf_sketch_api_config_file, $json);

        return $this->run_command($command);
    }

    public function getAllSketch($filter = null)
    {

        $request = new sketchRequestCommand();
        $request->searchAll($filter);
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $data = $apiArray['api_ok']['data']['search'];
        $sketches = array();
        reset($data);
        $sketchKey = key($data);
        $sketches = $data[$sketchKey];


        foreach ($sketches as $key => $sketch)
        {


            foreach ($sketch as $property => $v)
            {
                if ($property != 'metadata')
                {
                    unset($sketches[$key][$property]);
                }
            }
        }

        return $sketches;
    }

    public function getSketchInformation($sketch)
    {
        $request = new sketchRequestCommand();
        $request->getSketchInformation($sketch);
        $response = $this->run($request);
        $response = json_decode($response, true);
        $data = $response['api_ok']['data']['search'];
        $respond = array();


        foreach ($data as $key => $sketchList)
        {
            if (array_key_exists($sketch, $sketchList) !== false)
            {
                $respond = $sketchList[$sketch];
            }
        }

        return $respond;
    }

    public function getSketchSourcePath($sketch)
    {
        $request = new sketchRequestCommand();
        $request->getSketchInformation($sketch);
        $response = $this->run($request);
        $response = json_decode($response, true);
        $data = $response['api_ok']['data']['search'];
        $respond = "";


        foreach ($data as $key => $sketchList)
        {
            if (array_key_exists($sketch, $sketchList) !== false)
            {
                $respond = $key;
            }
        }

        return $respond;
    }

    public function getActivations($sketchName = null)
    {
        $request = new sketchRequestCommand();
        $request->getActivations();
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $data = $apiArray['api_ok']['data']['activations'];
        if ($sketchName)
        {
            if (array_key_exists($sketchName, $data))
            {
                $data = $data[$sketchName];
            }
            else
            {
                $data = array();
            }
        }

        return $data;
    }

    public function activate($sketchName, $target, $environment, $bundle, $params, $metadata = array())
    {
        $request = new sketchRequestCommand();
        $request->activateSketch($sketchName, $target, $environment, $bundle, array(
            $params
        ) , $metadata);
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $respond = $apiArray['api_ok']['data']['activate'];

        return $respond;
    }

    public function install($sketchName, $source, $target, $force)
    {
        $request = new sketchRequestCommand();
        $request->install($sketchName, $source, $target, $force = true);
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $respond = $apiArray['api_ok']['data']['install'];

        return $respond;
    }

    public function uninstall($sketchName, $target)
    {
        $request = new sketchRequestCommand();
        $request->uninstall($sketchName, $target);
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $respond = $apiArray['api_ok']['data']['uninstall'];

        return $respond;
    }

    public function defineParams($sketchName, $paramName, $params)
    {
        $request = new sketchRequestCommand();
        $request->defineParam($sketchName, $paramName, $params);
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $respond = $apiArray['api_ok']['data']['define'];

        return $respond;
    }

    public function undefineParams($paramNameArray = array())
    {
        $request = new sketchRequestCommand();
        $request->undefineParam($paramNameArray);
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $respond = $apiArray['api_ok']['data']['undefine'];

        return $respond;
    }

    public function defineEnvironment($envName, $environment)
    {
        $request = new sketchRequestCommand();
        $request->defineEnvironment($envName, $environment);
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $respond = $apiArray['api_ok']['data']['define_environment'];

        return $respond;
    }

    public function undefineEnvironment($environments = array())
    {
        $request = new sketchRequestCommand();
        $request->undefineEnvironment($environments);
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $respond = $apiArray['api_ok']['data']['undefine_environment'];

        return $respond;
    }

    public function getDefinitions()
    {
        $request = new sketchRequestCommand();
        $request->getDefinitions();
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $respond = $apiArray['api_ok']['data']['definitions'];

        return $respond;
    }

    public function getEnvironments()
    {
        $request = new sketchRequestCommand();
        $request->getEnvironments();
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $respond = $apiArray['api_ok']['data']['environments'];

        return $respond;
    }

    public function regenerate()
    {
        $request = new sketchRequestCommand();
        $request->regenerate();

        return $this->run($request);
    }

    public function getValidations($key = null)
    {
        $request = new sketchRequestCommand();
        $request->getValidations();
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $result = $apiArray['api_ok']['data']['validations'];
        if (!empty($key))
        {
            $returnResult = array();
            $explodeKeys = explode(',', $key);
            foreach($explodeKeys as $keys) {
                if (array_key_exists($keys, $result)) {
                    $returnResult[$keys] = $result[$keys];
                }
            }
            $result = $returnResult;

        }

        return $result;
    }

    public function validate($type, $data)
    {
        $request = new sketchRequestCommand();
        $request->validate($type, $data);
        $apiResponse = $this->run($request);

        return json_decode($apiResponse, true);
    }

    public function deactivate($id, $sketchName, $target)
    {

        $activationDetails = $this->get_activation_details($id, $sketchName);

        $request = new sketchRequestCommand();
        $request->deactivate($id);
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $respond = $apiArray['api_ok']['data']['deactivate'];
        $this->_cleanupSketchDataAfterDeactivation($activationDetails, $target, $sketchName);

        return $respond;
    }
    /**
     * Remove parameters, environments and uninstall sketch if no activation is there.
     * @param  array $activationDetails
     * @return boolean
     */
    function _cleanupSketchDataAfterDeactivation($activationDetails, $target, $sketchName = null)
    {
        // remove parameters
        if (!empty($activationDetails['params']))
        {
            $this->undefineParams($activationDetails['params']);
        }
        // remove parameters
        if (!empty($activationDetails['environment']))
        {
            // check if environment is being used in other activations as well
            if (!$this->checkActivationEnvironment($activationDetails['identifier'], $activationDetails['environment']))
            {
                $env = array(
                    $activationDetails['environment']
                );
                $this->undefineEnvironment($env);
            }
        }

        $activations = $this->getActivations($sketchName);
        if (empty($activations))
        {
            $this->uninstall($sketchName, $target);
        }

        return true;
    }
    /**
     * get details of activation with id as parameter
     * @param  string $activationId unique identifier for the activation
     * @return array  array with activation details as params,environment else empty if not  found
     */
    function get_activation_details($activationId, $sketchName)
    {
        $allActivations = $this->getActivations($sketchName);

        foreach ($allActivations as $activation)
        {
            if (!empty($activation['identifier']) && $activation['identifier'] == $activationId)
            {
                        return $activation;
            }
        }

        return array();
    }
    /**
     * check if the environment is used in other activations as well
     * @param  string $activationId Activation id
     * @param  string $environment  Environment to check for
     * @return bool                 true if used else false
     */
    function checkActivationEnvironment($activationId, $environment)
    {
        $allActivations = $this->getActivations();


        foreach ((array)$allActivations as $sketch => $activations)
        {


            foreach ($activations as $activation)
            {
                if (!empty($activation['environment']) && ($activation['environment'] == $environment) && $activation['identifier'] != $activationId)
                {

                    return true;
                }
            }
        }

        return false;
    }
}
?>