<?php


/**
 * @uri /health-diagnostic/status
 */
class HealthStatus extends CfProtectedResource
{
    public function get($request)
    {
        $model = new HealthDiagnosticModel($this->username);

        $response = new Response($request);
        $response->body = json_encode($model->getHealthStatus());
        $response->code = Response::OK;

        return $response;
    }
}

/**
 * @uri /health-diagnostic/report_ids
 */
class HealthReports extends CfProtectedResource
{
    public function get($request)
    {
        $response = new Response($request);
        $response->body = json_encode(BaseHealthDiagnostic::$reportTypes);
        $response->code = Response::OK;

        return $response;
    }
}

/**
 * @uri /health-diagnostic/report/:reportName
 */
class HealthReport extends CfProtectedResource
{
    public function post($request, $reportName)
    {
        $response = new Response($request);
        $model = new HealthDiagnosticReportModel($this->username, $reportName);
        $data = Utils::getValidJsonData($request->data);

        $query = $model->getSql();

        if (!isset($data->sortDescending)) {
            $data->sortDescending = false;
        }

        if (isset($data->skip) && !is_int($data->skip)) {
            $response->code = Response::BADREQUEST;
            $response->body = "'skip' field must be an integer";
            return $response;
        } else {
            $data->skip = -1;
        }

        if (isset($data->limit) && !is_int($data->limit)) {
            $response->code = Response::BADREQUEST;
            $response->body = "'limit' field must be an integer";
            return $response;
        } else {
            $data->limit = -1;
        }

        if (isset($data->hostContextInclude) && !is_array($data->hostContextInclude)) {
            $response->code = Response::BADREQUEST;
            $response->body = "'hostContextInclude' field must be an array";
            return $response;
        } else {
            $data->hostContextInclude = [];
        }

        if (isset($data->hostContextExclude) && !is_array($data->hostContextExclude)) {
            $response->code = Response::BADREQUEST;
            $response->body = "'hostContextExclude' field must be an array";
            return $response;
        } else {
            $data->hostContextExclude = [];
        }

        $response->body = cfapi_query_post($this->username,
            $query,
            $data->sortColumn,
            $data->sortDescending,
            $data->skip,
            $data->limit,
            $data->hostContextInclude,
            $data->hostContextExclude);
        $response->code = Response::OK;

        return $response;
    }
}

/**
 * @uri /health-diagnostic/dismiss/:reportName
 */
class HealthDismissed extends CfProtectedResource
{
    private $model;

    public function __construct($parameters)
    {
        parent::__construct($parameters);
        $this->model = new DismissHealthDiagnosticModel(CfdbPdo::getInstance()->getConnection(), $this->username);
    }

    public function get($request, $reportName)
    {
        $limit = Utils::queryParam('limit');
        $offset = Utils::queryParam('offset');
        $response = new Response($request);
        $response->body = $this->model->dismissedList($reportName, $limit, $offset);
        $response->code = Response::OK;
        return $response;
    }

    public function post($request, $reportName)
    {
        $data = Utils::getValidJsonData($request->data);
        $response = new Response($request);

        if (isset($data->hosts) && !is_array($data->hosts)) {
            $response->code = Response::BADREQUEST;
            $response->body = "'hosts' field must be an array";
            return $response;
        }

        $this->model->add($data->hosts, $reportName);
        $response->code = Response::CREATED;
        return $response;

    }

    public function delete($request, $reportName)
    {
        $data = Utils::getValidJsonData($request->data);
        $response = new Response($request);

        if (isset($data->hosts) && !is_array($data->hosts)) {
            $response->code = Response::BADREQUEST;
            $response->body = "'hosts' field must be an array";
            return $response;
        }

        $this->model->delete($data->hosts, $reportName);
        $response->code = Response::ACCEPTED;
        return $response;
    }
}
