<?php
abstract class SketchBase extends CfProtectedResource
{
  public $dcSettings = null;

  public function __construct($parameters = array())
  {
    parent::__construct($parameters);
    $this->dcSettings = new DcSettings();
    $DcConfiguration['cf_sketch_api_config_file'] = $this->dcSettings->get_dc_user_sketch_config_file_path($this->username);
    $DcConfiguration['cf_sketch_api_executable'] = $this->dcSettings->get_dc_sketch_api_executable();
    $DcConfiguration['cf_sketch_api_script_path'] = $this->dcSettings->get_dc_user_working_directory($this->username);
    $this->sketchApi = new SketchApi($DcConfiguration);
    $userSettings = new stdClass();
    $userSettings->username = $this->username;
    $userSettings->gitServer = $this->dcSettings->get_dc_staging_repository();
    $userSettings->gitBranch = $this->dcSettings->get_dc_git_working_branch();
    $this->dcApi = new DcApi($userSettings);
  }

  public function _filterSketchesRBAC($allSketches = array())
  {

    $isAdmin = false;
    $userRbacRoles = utils::get_rbac_settings($this->username);
    syslog(LOG_DEBUG, "Applying RBAC filter to the sketches for user $this->username");
    $rbacMode = $userRbacRoles['rbac'];
    $messageRbac = ($rbacMode) ? "on" : "off";

    if (!$rbacMode)
    {
      return $allSketches;
    }

    if (!empty($userRbacRoles['roles']))
    {
      // check for admin role
      $assignedRoles = $userRbacRoles['roles'];
      if (in_array('admin', $assignedRoles))
      {
        $isAdmin = true;
        syslog(LOG_DEBUG, "$this->username is in admin role so not filtering any sketches");
      }
    }
    if (!$isAdmin)
    {
      $sketchRBACJson = cfapi_user_sketches_list($this->username, $this->username);
      $sketchAvailable = array();

      if (!empty($sketchRBACJson))
      {
        $sketchRBAC = json_decode($sketchRBACJson, true);


        foreach ($allSketches as $key => $sketch)
        {
          if (in_array($key, $sketchRBAC['data']) === true)
          {
            $sketchAvailable[$key] = $sketch;
          }
        }
      }
    }
    else
    {
      $sketchAvailable = $allSketches;
    }

    return $sketchAvailable;
  }
  /**
   * This function will split the context into includes and excludes applied by RBAC for querying the hosts.
   * @param  [type] $value [description]
   * @return [type]        [description]
   */
  public function _splitRbacRules($includeregex = array())
  {

    $excludeArray = array();
    $removeExcludes = array();


    foreach ($includeregex as $val)
    {
      // find the exlude pattern eg: !(someclass)
      if (preg_match('/^!\((.*)\)$/', $val, $matches))
      {
        $excludeArray[] = $matches[1];
        $removeExcludes[] = $matches[0];
      }
    }
    $includes = array_diff($includeregex, $removeExcludes);
    $result = array(
      'includes' => $includes,
      'excludes' => $excludeArray
    );
    return $result;
  }

  public function _getActivationHostDetails($includeregex = array() , $hash)
  {
    $returnObject = array();

    try
    {

      $query = sprintf("select DISTINCT h.hostkey,h.hostname,p.promiseoutcome
              from hosts h LEFT outer join promiseexecutions p
              on h.hostkey = p.hostkey
              and '{\"%s\"}' <@ p.promisees
              and p.bundleName = 'cfsketch_run'", $hash);

      $context = $this->_splitRbacRules($includeregex);
      $includes = $context['includes'];
      $excludes = $context['excludes'];

      $resultJson = cfapi_query_post($this->username, $query, $sortColumn = '', $sortDescending = '', $skip = 0, $limit = - 1, $includes, $excludes);
      $result = json_decode($resultJson, true);
      $rows = empty($result['data'][0]['rows']) === false ? $result['data'][0]['rows'] : array();
      // format the results


      foreach ($rows as $host)
      {
        $obj = new stdClass();
        $obj->key = $host[0];
        $obj->name = $host[1];
        $status = empty($host[2]) || $host[2] == "NULL" ? "warning" : strtolower($host[2]);
        if ($status === "repaired")
        {
          // treat repaired as kept
          $status = "kept";
        }
        $obj->status = $status;
        $returnObject[] = $obj;
      }
    }

    catch(Exception $e)
    {
      syslog(LOG_ERR, $e->getMessage());
      throw $e;
    }

    return $returnObject;
  }

  public function _activation_details_get($activation, $sketchInfo)
  {
    $sketchName = $sketchInfo['metadata']['name'];
    $activationDetailsStructure = new stdClass();
    $activationDetailsStructure->params = array();
    $activationDetailsStructure->environments = array();
    $activationDetailsStructure->hosts = array();
    $activationDetailsStructure->promiseHandle = '';
    if ($activation['identifier'])
    {
      $environment = $activation['environment'];
      $definitions = $activation['params'];
      $hash = $activation['hash'];
      $availableDefinations = $this->sketchApi->getDefinitions();
      // assume only one variable is defined for now
      $paramDefination = $definitions[0];
      if (!empty($availableDefinations[$paramDefination]))
      {
        $params = $availableDefinations[$paramDefination][$sketchName];
        $activationDetailsStructure->params = $params;
      }

      $availableEnvironment = $this->sketchApi->getEnvironments();
      if (!empty($availableEnvironment[$environment]))
      {
        $activationDetailsStructure->environments = $availableEnvironment[$environment];
        $sketchName = $sketchInfo['metadata']['name'];
        $namespace = $sketchInfo['namespace'];
        $includeregex = !empty($activationDetailsStructure->environments['activated']['include']) ? $activationDetailsStructure->environments['activated']['include'] : array();
        $activationDetailsStructure->hosts = $this->_getActivationHostDetails($includeregex, $hash);
      }
    }

    return $activationDetailsStructure;
  }
}
/**
 * @uri /dc/sketch
 */
class SketchList extends SketchBase
{
  public function get($request)
  {

    syslog(LOG_DEBUG, 'Requesting GET /api/dc/sketch');

    $response = new Response($request);
    $filter = $this->dcSettings->get_dc_sketch_filter();
    if (Utils::queryParam('name'))
    {
      $sketchName = trim(urldecode(Utils::queryParam('name')));
      array_push($filter, array(
        'name',
        'matches',
        $sketchName
      ));
    }

    $allSketches = $this->sketchApi->getAllSketch($filter);
    $sketchAvailable = $this->_filterSketchesRBAC($allSketches);
    $response->body = utils::packageResult($sketchAvailable, 1, count($sketchAvailable));
    $response->code = Response::OK;

    return $response;
  }
}
/**
 * @uri /dc/sketch/:name 1
 */
class Sketch extends SketchBase
{
  public function get($request, $name)
  {
    syslog(LOG_DEBUG, 'Requesting GET /api/dc/sketch/:name');

    $response = new Response($request);
    $filter = $this->dcSettings->get_dc_sketch_filter();
    $sketchName = trim(urldecode($name));
    $sketchAvailable = array();
    $found = 0;
    if (!empty($sketchName))
    {
      $sketchAvailable = $this->sketchApi->getSketchInformation($sketchName);
      if (!empty($sketchAvailable))
      {
        $result = array(
          $sketchAvailable
        );
        $response->body = utils::packageResult($result, 1, 1);
        $response->code = Response::OK;
      }
      else
      {
        $response->body = null;
        $response->code = Response::NOTFOUND;
      }
    }

    return $response;
  }

  public function put($request, $name = null)
  {
    syslog(LOG_DEBUG, 'Requesting PUT /api/dc/sketch/:name');

    $sketchName = urldecode($name);
    $response = new Response($request);
    $sourcePath = $this->sketchApi->getSketchSourcePath($sketchName);
    $target = $this->dcSettings->get_dc_user_repolist_location($this->username);
    $install = $this->sketchApi->install($sketchName, $sourcePath, $target, $force = true);
    $response->body = json_encode($install);
    $response->code = Response::OK;

    return $response;
  }
}
/**
 * @uri /dc/activation 0
 */
class SketchActivationList extends SketchBase
{
  public function get($request)
  {
    syslog(LOG_DEBUG, 'Requesting GET /api/dc/activation');
    $response = new Response($request);
    $sketchName = null;
    if (Utils::queryParam('sketch'))
    {
      $sketchName = urldecode(Utils::queryParam('sketch'));
    }

    $activations = $this->sketchApi->getActivations($sketchName);
    $details = (bool)Utils::queryParam('details');
    $sketchActivation = array();
    if ($details && !empty($sketchName) && !empty($activations))
    {

      $sketchActivation = $activations;
      $sketchAvailable = $this->sketchApi->getSketchInformation($sketchName);

      try
      {


        foreach ($activations as $index => $activation)
        {
          $details = $this->_activation_details_get($activation, $sketchAvailable);
          $activations[$index]['details'] = $details;
        }
      }
      catch(Exception $e)
      {
        syslog(LOG_ERR, $e->getMessage());
        $response->body = json_encode($e->getMessage());
        $response->code = Response::INTERNALSERVERERROR;
        return $response;
      }
    }

    $response->body = utils::packageResult($activations, 1, count($activations));
    $response->code = Response::OK;
    return $response;
  }
}
/**
 * @uri /dc/activation/:id/([^\/]*) 1
 */
class SketchActivation extends SketchBase
{
  public function get($request, $id, $sketch)
  {
    syslog(LOG_DEBUG, 'Requesting GET /api/dc/activation/:id/:sketch');
    try
    {
      $response = new Response($request);
      $activationIdentifier = urldecode($id);
      $sketchName = urldecode($sketch);
      $activation = $this->sketchApi->get_activation_details($activationIdentifier, $sketchName);
      $details = (bool)Utils::queryParam('details');
      if ($details && !empty($activation))
      {
        $sketchAvailable = $this->sketchApi->getSketchInformation($sketchName);
        $details = $this->_activation_details_get($activation, $sketchAvailable);
        $activation['details'] = $details;
      }

      $result = array(
        $activation
      );
      $response->body = utils::packageResult($result, 1, 1);
      $response->code = Response::OK;
    }
    catch(Exception $e)
    {
      syslog(LOG_ERR, $e->getMessage());
      $response->body = json_encode($e->getMessage());
      $response->code = Response::INTERNALSERVERERROR;
    }

    return $response;
  }

  public function put($request, $id, $sketch)
  {
    syslog(LOG_DEBUG, 'Requesting PUT /api/dc/activation');

    $id = urldecode($id);
    $data = Utils::getValidJsonData($request->data, true);

    if (!preg_match('/^[a-z0-9_\- ]+$/i', $id))
    {
      $message = "Sketch activation name can only contain alpha numeric with exception of spaces - or _  characters";
      throw new Exception($message);
    }

    $requiredParameters = array('environmentName','bundleName','paramName');
    Utils::checkRequiredParameters($data,$requiredParameters);

    $target = $this->dcSettings->get_dc_user_repolist_location($this->username);
    $sketchName = urldecode($sketch);
    $envName = $data['environmentName'];
    $bundleName = $data['bundleName'];
    $paramName = $data['paramName'];
    $extraData = !empty($data['metaData']) ? $data['metaData'] : array();
    $metaData = array(
      'identifier' => $id,
      'timestamp' => time()
    );

    $metaData = array_merge($metaData, $extraData);

    $checkAccess = array(
      $sketchName => $sketchName
    );
    $hasAccess = $this->_filterSketchesRBAC($checkAccess);

    if (empty($hasAccess))
    {
      $response = new Response($request);
      $message = "Permission denied to activate the sketch.";
      $response->body = $message;
      $response->code = Response::UNAUTHORIZED;
      return $response;
    }

    $checkSameActivation = $this->sketchApi->get_activation_details($metaData['identifier'], $sketchName);

    if (!empty($checkSameActivation))
    {
      $response = new Response($request);
      $message = "Sketch activation with same name already exists.";
      $response->body = $message;
      $response->code = Response::BADREQUEST;
      return $response;
    }

    $activate = $this->sketchApi->activate($sketchName, $target, $envName, $bundleName, $paramName, $metaData);
    $regenerate = $this->sketchApi->regenerate();
    $response = new Response($request);
    $response->body = json_encode($activate);
    $response->code = Response::OK;

    return $response;
  }

  public function delete($request, $id, $sketch)
  {
    syslog(LOG_DEBUG, 'Requesting DELETE /api/dc/activation');

    $id = urldecode($id);
    $sketchName = urldecode($sketch);
    $checkAccess = array(
      $sketchName => $sketchName
    );
    $hasAccess = $this->_filterSketchesRBAC($checkAccess);

    if (empty($hasAccess))
    {
      $response = new Response($request);
      $message = "Permission denied to activate the sketch.";
      $response->body = $message;
      $response->code = Response::UNAUTHORIZED;
      return $response;
    }

    $target = $this->dcSettings->get_dc_user_repolist_location($this->username);
    $deactivate = $this->sketchApi->deactivate($id, $sketchName, $target);
    $regenerate = $this->sketchApi->regenerate();
    $response = new Response($request);
    $response->body = json_encode($deactivate);
    $response->code = Response::OK;

    return $response;
  }
}
/**
 * @uri /dc/definition 0
 */
class SketchDefinitionList extends SketchBase
{
  public function get($request)
  {

    syslog(LOG_DEBUG, 'Requesting GET /api/dc/definition');

    $response = new Response($request);
    $definitions = $this->sketchApi->getDefinitions();
    $response->body = utils::packageResult($definitions, 1, count($definitions));
    $response->code = Response::OK;

    return $response;
  }
}
/**
 * @uri /dc/definition/:paramName 0
 */
class SketchDefinition extends SketchBase
{
  public function put($request, $paramName)
  {
    syslog(LOG_DEBUG, 'Requesting PUT /api/dc/definition/:paramName');

    $paramName = urldecode($paramName);

    $data = Utils::getValidJsonData($request->data, true);

    $requiredParameters = array('sketchName','params');
    Utils::checkRequiredParameters($data,$requiredParameters);

    $sketchName = urldecode($data['sketchName']);
    $params = $data['params'];

    if (!preg_match('/^[a-z0-9_\-]+$/i', $paramName))
    {
      $message = "Parameter name can only contain alpha numeric with exception of - or _  characters";
      throw new Exception($message);
    }

    $validSketch = $this->sketchApi->getSketchInformation($sketchName);

    if (empty($validSketch))
    {
      $message = sprintf("Sketch named %s not found.", $sketchName);
      throw new Exception($message);
    }

    $apiResponse = $this->sketchApi->defineParams($sketchName, $paramName, $params);
    $response = new Response($request);
    $response->body = json_encode($apiResponse);
    $response->code = Response::OK;

    return $response;
  }

  public function post($request, $paramName)
  {
    syslog(LOG_DEBUG, 'Requesting PUT /api/dc/definition/:paramName');

    $data = Utils::getValidJsonData($request->data, true);

    $requiredParameters = array('sketchName','params');
    Utils::checkRequiredParameters($data,$requiredParameters);


    $sketchName = urldecode($data['sketchName']);
    $params = $data['params'];
    $apiResponse = $this->sketchApi->defineParams($sketchName, $paramName, $params);
    $response = new Response($request);
    $response->body = json_encode($apiResponse);
    $response->code = Response::OK;

    return $response;
  }
}
/**
 * @uri /dc/environment 1
 */
class SketchEnvironmentList extends SketchBase
{
  public function get($request)
  {
    syslog(LOG_DEBUG, 'Requesting GET /api/dc/environment');

    $response = new Response($request);
    $environments = $this->sketchApi->getEnvironments();
    $response->body = utils::packageResult($environments, 1, count($environments));
    $response->code = Response::OK;

    return $response;
  }
}
/**
 * @uri /dc/environment/:environmentName 0
 */
class SketchEnvironment extends SketchBase
{
  public function put($request, $environmentName)
  {
    syslog(LOG_DEBUG, 'Requesting PUT /api/dc/environment/:environmentName');

    $data = Utils::getValidJsonData($request->data, true);

    $requiredParameters = array('environment');
    Utils::checkRequiredParameters($data,$requiredParameters);

    if (!preg_match('/^[a-z0-9_\-]+$/', $environmentName))
    {
      throw new Exception("Environment name can only contain alpha numeric with exception of - or _  characters");
    }

    $params = $data['environment'];
    $includes = $params;
    $excludes = array();
    // RBAC check
    $userRbac = utils::get_rbac_settings($this->username);
    if ($userRbac['rbac'] && !in_array('admin', $userRbac['roles']))
    {
      $excludes = array();


      foreach ($userRbac['excludes'] as $e)
      {
        $excludes[] = '!(' . $e . ')'; // negate the exclude rule


      }
      $includes = array_merge($includes, $userRbac['includes'], $excludes);
      syslog(LOG_DEBUG, 'Applying RBAC settings in sketch environment with environment.' . json_encode($includes));
    }

    $apiResponse = $this->sketchApi->defineEnvironment($environmentName, $includes);
    $response = new Response($request);
    $response->body = json_encode($apiResponse);
    $response->code = Response::OK;

    return $response;
  }
}
/**
 * @uri /dc/validation 1
 */
class SketchValidationList extends SketchBase
{
  public function get($request)
  {
    syslog(LOG_DEBUG, 'Requesting GET /api/dc/validation');

    $filter = Utils::queryParam('filter');
    $response = new Response($request);
    $validations = $this->sketchApi->getValidations($filter);
    $response->body = utils::packageResult($validations, 1, count($validations));
    $response->code = Response::OK;

    return $response;
  }
}
/**
 * @uri /dc/validation/:key 0
 */
class SketchValidation extends SketchBase
{
  public function get($request, $key)
  {
    syslog(LOG_DEBUG, 'Requesting GET /api/dc/validation/:key');

    $response = new Response($request);
    $validations = $this->sketchApi->getValidations($key);
    $response->body = utils::packageResult($validations, 1, count($validations));
    $response->code = Response::OK;

    return $response;
  }
}
/**
 * @uri /dc/validate/:validationType 0
 */
class SketchValidate extends SketchBase
{
  public function post($request, $validationType)
  {

    syslog(LOG_DEBUG, 'Requesting POST /api/dc/validate/:validationType');

    $validationData = Utils::getValidJsonData($request->data, true);
    $response = new Response($request);
    $validations = $this->sketchApi->validate($validationType, $validationData);
    $response->body = json_encode($validations);
    $response->code = Response::OK;

    return $response;
  }
}
