<?php

require_once "APIBaseTest.php";

class RoleTest extends APIBaseTest
{
    public function testListRoles()
    {
        try
        {
            $jsonArray = $this->getResults('/role');
            $this->assertValidJson($jsonArray);
            $this->assertEquals(1, sizeof($jsonArray));
        }
        catch (Pest_Exception $e)
        {
            $this->fail($e);
        }
    }

    public function testInvalidRolesDetail()
    {
        try
        {
            $jsonArray = $this->getResults('/role/xxx');
            $this->fail('should throw 404 not found exception');
        }
        catch (Pest_Exception $e)
        {
           $this->assertEquals(404, $this->pest->lastStatus());
        }
    }

    public function testAddRole(){
        try
        {
             $this->pest->put('/role/jersey', '{
                     "description": "Jersey Shore Role",
                     "includeContext":"linux",
                     "excludeContext":"windows"
                }');
            $this->assertEquals(201, $this->pest->lastStatus());

            // check role was added
            $roles = $this->getResults('/role/jersey');

            $this->assertValidJson($roles);
            $this->assertEquals('Jersey Shore Role', $roles[0]['description']);
            $this->assertEquals('jersey', $roles[0]['id']);
            $this->assertEquals('linux', $roles[0]['includeContext']);
            $this->assertEquals('windows', $roles[0]['excludeContext']);

        }
        catch(Pest_Exception $e)
        {
            $this->fail($e);
        }
    }

    public function testAddRoleWithSketches()
    {
        try
        {
            $this->pest->put('/role/jersey', '{
                     "description": "Jersey Shore Role",
                     "includeContext":"linux",
                     "excludeContext":"windows",
                     "includeSketches":["s1","s2"]
                }');
            $this->assertEquals(201, $this->pest->lastStatus());

            // check role was added
            $roles = $this->getResults('/role/jersey');


            $this->assertValidJson($roles);
            $this->assertEquals('Jersey Shore Role', $roles[0]['description']);
            $this->assertEquals('jersey', $roles[0]['id']);
            $this->assertEquals('linux', $roles[0]['includeContext']);
            $this->assertEquals('windows', $roles[0]['excludeContext']);
            $this->assertEquals('s1', $roles[0]['sketches'][0]);
            $this->assertEquals('s2', $roles[0]['sketches'][1]);

        }
        catch(Pest_Exception $e)
        {
            $this->fail($e);
        }
    }

    public function testAddRoleWithEmptySketches()
    {
        try
        {
            $this->pest->put('/role/jersey', '{
                     "description": "Jersey Shore Role",
                     "includeContext":"linux",
                     "excludeContext":"windows",
                     "includeSketches":[]
                }');
            $this->assertEquals(201, $this->pest->lastStatus());

            // check role was added
            $roles = $this->getResults('/role/jersey');


            $this->assertValidJson($roles);
            $this->assertEquals('Jersey Shore Role', $roles[0]['description']);
            $this->assertEquals('jersey', $roles[0]['id']);
            $this->assertEquals('linux', $roles[0]['includeContext']);
            $this->assertEquals('windows', $roles[0]['excludeContext']);
            $this->assertTrue(empty($roles[0]['sketches']));

        }
        catch(Pest_Exception $e)
        {
            $this->fail($e);
        }
    }

    public function testEditRole()
    {
        try
        {
            $this->pest->post('/role/admin', '{
                    "description": "admin modified",
                    "crxi":"linux,windows"
                }');
            $this->assertEquals(204, $this->pest->lastStatus());

            // check role was added
            $roles = $this->getResults('/role');
            $this->assertValidJson($roles);
            $this->assertEquals('admin', $roles[0]['id']);
            $this->assertEquals('admin modified', $roles[0]['description']);
        }
        catch (Pest_Exception $e)
        {
            $this->fail($e);
        }
    }

    public function testEditRoleWithSketches()
    {
        try
        {
            $this->pest->post('/role/admin', '{
                    "description": "admin modified",
                    "crxi":"linux,windows",
                    "includeSketches":["s1","s2"]
                }');
            $this->assertEquals(204, $this->pest->lastStatus());

            // check role was added
            $roles = $this->getResults('/role');
            $this->assertValidJson($roles);
            $this->assertEquals('admin', $roles[0]['id']);
            $this->assertEquals('admin modified', $roles[0]['description']);
            $this->assertEquals('s1', $roles[0]['sketches'][0]);
            $this->assertEquals('s2', $roles[0]['sketches'][1]);

        }
        catch (Pest_Exception $e)
        {
            $this->fail($e);
        }
    }

    public function testEditRoleWithNullSketches()
    {
        try
        {
            $this->pest->post('/role/admin', '{
                    "description": "admin modified",
                    "crxi":"linux,windows",
                    "includeSketches":[]
                }');
            $this->assertEquals(204, $this->pest->lastStatus());

            // check role was added
            $roles = $this->getResults('/role');
            $this->assertValidJson($roles);
            $this->assertEquals('admin', $roles[0]['id']);
            $this->assertEquals('admin modified', $roles[0]['description']);
            $this->assertTrue(empty($roles[0]['sketches']));

        }
        catch (Pest_Exception $e)
        {
            $this->fail($e);
        }
    }


    public function testDeleteRole()
    {
         // delete role
        try
        {
            $this->pest->delete('/role/admin');
            $this->assertEquals(204, $this->pest->lastStatus());
        }
        catch (Pest_Exception $e)
        {
            $this->fail($e);
        }
    }


    public function testAddRoleWithInvalidData()
    {
        try
        {
             $this->pest->put('/role/jersey', '{
                     "description": "Jersey Shore Role"
                     "includeContext":"linux,
                      excludeContext":"windows"
                }');
             $this->fail('Should trigger exception');

        }
        catch(Pest_Exception $e)
        {
            $this->assertEquals(400, $this->pest->lastStatus());
        }
    }

    public function testUpdateRoleWithInvalidData()
    {
        try
        {
             $this->pest->put('/role/admin', '{
                     "description": "INVALID JSON"
                     "includeContext":"linux,
                      excludeContext":"windows"
                }');
             $this->fail('Should trigger exception');

        }
        catch(Pest_Exception $e)
        {
            $this->assertEquals(400, $this->pest->lastStatus());
        }
    }

}
