<?php
require_once APPPATH . 'libraries/Cf_REST_Controller.php';

require_once APPPATH . 'modules/dashboard/models/entities/CF_Widget.php';
require_once APPPATH . 'modules/dashboard/models/entities/CF_Alert.php';
require_once APPPATH . 'modules/dashboard/models/entities/CF_Rule.php';

class Alertsapi extends Cf_REST_Controller
{

    public $username = "";
    private $roles = null;
    private $availableSketches = array();

    function __construct()
    {
        parent::__construct();

        $this->load->model('alerts_model');
        $this->load->driver('cache', array('adapter' => 'file'));
        $this->username = $this->session->userdata('username');
    }


    //$alertsIds - can be urlencoded comma separated list
    function alerts_get($alertsIds = '')
    {

        $filter = array();
        //$filter['username'] = $this->session->userdata('username');

        if (!empty($alertsIds))
        {
            $alertsIds = explode(',', urldecode($alertsIds));
            $alertsIds = array_map("intval",$alertsIds);
        }

        try
        {
            $result = $this->alerts_model->get_alerts_with_ids($alertsIds, $filter);
            if (empty($result)) {
                $message = $this->lang->line('dashboard_alert_not_exist');
                log_message('debug', 'Failed retrieving alerts, error: ' . $message);
                respond_internal_error($message);
                return;
            }

            $json = json_encode($result);
            respond_ok($json);
        }
        catch(Exception $e)
        {
            $message = $e->getMessage();
            log_message('debug', 'Failed retrieving alerts, error: ' . $message);
            respond_internal_error($message);
        }
    }

    /**
     * Return alert + rule
     *
     *
     * @param type $alertsIds
     */
    function alertswithrule_get($alertsIds = '')
    {

        $filter = array();
        //$filter['username'] = $this->session->userdata('username');

        if (!empty($alertsIds))
        {
            $alertsIds = explode(',', urldecode($alertsIds));
            $alertsIds = array_map("intval",$alertsIds);
        }

        try
        {
            $alertsList = $this->alerts_model->get_alerts_with_ids($alertsIds, $filter, array('name' => 'asc'));

            if(empty($alertsList))
            {
                respond_ok();
            }

            $rulesIds = array();
            foreach($alertsList as $item => $alertObj)
            {
                $ruleId = $alertObj->getRuleId();
                if (!empty($ruleId))
                {
                    $rulesIds[] = $ruleId;
                }
            }

            // return result without rules info
            if (empty($rulesIds))
            {
                $json = json_encode($alertsList);
                respond_ok($json);
            }

            $this->load->model('rules_model');
            $userFilter = array();
            //$userFilter = array('username' => $this->session->userdata('username'));
            $rulesTmp = $this->rules_model->get_rules_with_ids(array_values($rulesIds),$userFilter);

            // TODO: discuss - this is error, because in alert we have reference to the rule, but rule doesn't exist
            if (empty($rulesTmp))
            {
                $json = json_encode($alertsList);
                respond_ok($json);
            }


            // join data together
            // create array where rule.id is a key
            $rules = array();
            foreach($rulesTmp as $item => $ruleObj)
            {
                $rules[$ruleObj->id] = $ruleObj;
            }


            $result = array();
            $i = 0;
            foreach($alertsList as &$alertObj)
            {
                $result[$i] = (array)$alertObj;
                if (!empty($alertObj->ruleid) && isset($rules[$alertObj->ruleid]))
                {
                    $result[$i]['rule'] = (array)$rules[$alertObj->ruleid];
                }
                $i++;
            }

            $json = json_encode($result);
            respond_ok($json);
        }
        catch(Exception $e)
        {
            $message = $e->getMessage();
            log_message('debug', 'Failed retrieving alerts, error: ' . $message);
            respond_internal_error($message);
        }
    }



    /**
     * Save alert
     * Allow overwrite only by id
     * same name not allowed  - show error
     *
     *
     * @param <string> $id - id of the alert. If not provided - assume insert, if provided - assume update
     * @return
     */
    function savealert_post($id = '')
    {

        $data = json_decode($this->request->body, true);

        $data['username'] = $this->session->userdata('username');
        $is_admin = $this->ion_auth->is_admin();


        $overwrite = FALSE;
        if (!empty($id))
        {
            $overwrite = TRUE;
        }

        try
        {
            $obj = $this->alerts_model->savealert($data, $overwrite, $is_admin);
        }
        catch(Exception $e)
        {
            respond_internal_error($e->getMessage());
        }

        if (!$obj) //not able to get inserted alert
        {
            $errors = $this->alerts_model->getErrors();
            $data = json_encode($errors);
            respond_internal_error($data);
            return;
        }
        else
        {
            $data = array();
            $data[0] = array('alertId' => $obj->id, 'text' => $this->lang->line('dashboard_alert_save_success'), 'errorCode' => 200);
            respond_ok(json_encode($data));
            return;
        }
    }

    function deletealert_delete($id) {

        $username = $this->session->userdata('username');
        $is_admin = $this->ion_auth->is_admin();

        try
        {
            $result = $this->alerts_model->deletealert($id, $username, $is_admin);
            $json = json_encode($result);
            respond_ok($json);
        }
        catch(Exception $e)
        {
            $message = $e->getMessage();
            log_message('debug', 'Unable to delete alert. Error: ' . $message);
            respond_internal_error($message);
        }
    }

    /**
     * Return list of alert hosts
     *
     * @param <string> $id - alertId
     */
    function getalerthosts_post($id) {
        $username = $this->session->userdata('username');
        $is_admin = $this->ion_auth->is_admin();

        $data = json_decode($this->request->body, true);
        $load_limit = $data['load_limit'];

        try
        {
            $result = $this->alerts_model->getalerthosts($id, $username, $is_admin, $load_limit);
            $json = json_encode($result);
            respond_ok($json);
        }
        catch(Exception $e)
        {
            $message = $e->getMessage();
            log_message('debug', 'Unable to delete alert. Error: ' . $message);
            respond_internal_error($message);
        }
    }

    /**
     * This function gets the includes for sketch activation required for the alert
     * @param  [type] $environment [description]
     * @return [type]              [description]
     */
    function getalertcontextforsketchactivation_get($environment)
    {

        $this->load->model('designcenter/dc_rest_model');
        $env = $this->dc_rest_model->getEnvironments();
        $data = array();
        foreach ($env['data'] as $key=>$envData)
        {
            if ($key == $environment) {
                $data = $envData['activated']['include'];
            }
        }
        $json = json_encode($data);

        respond_ok($json);

    }

    /**
     * Create widget, alert and rule for DC activation (called from DC)
     * @deprecated
     * @return json
     */
    function createAlertFromDC_post()
    {
        $username = $this->session->userdata('username');
        $data = json_decode($this->request->body, true);

        $this->load->model('settings_rest_model');
        $settingsValue = $this->settings_rest_model->get_app_settings();


        $sketchName     = $data['sketchName'];
        $activationName = $data['identifier'];
        $hash = $data['hash'];
        $includes = $data['includes'];

        $is_admin = $this->ion_auth->is_admin();

        //1. check if widget with "sketchName" already exists and create if not
        $widget_filter = array();
        $widget_filter['username'] = $username;
        $widget_filter['name']     = $sketchName;
        $widget_filter['type']     = 'alerts';


        $defaultAlertName = $this->_getDefaultAlertName_DC($activationName);
        $defaultRuleName  = $this->_getDefaultRuleName_DC($activationName);


        $widgetData = array();
        $alertData  = array();
        $ruleData   = array();

        $this->load->model('dashboard_model');
        $this->load->model('widget_model');
        $this->load->model('rules_model');

        $userDash = $this->dashboard_model->get_user_selection(array('username' => $username));
        if (!empty($userDash))
        {
            $dashboardId = $userDash[0]['dashboard'];
            $widget_filter['dashboardid'] = $dashboardId;
        }


        // get widget on current dashboard 

        $widget = $this->widget_model->get_all_widgets($widget_filter);

        $widgetFound = FALSE;
        $alertFound  = FALSE;
        //$ruleFound   = FALSE;


        if (empty($widget))
        {
            $widgetData = array();
            $widgetData['name']     = $sketchName;
            $widgetData['type']     = 'alerts';
            $widgetData['alerts']   = array();
            $widgetData['username'] = $username;
            $widgetData['dashboardid'] = $dashboardId;
        }
        else
        {
            $widgetFound = TRUE;
            $widgetData = (array)$widget[0];
        }

        // check if alert already exist, if not - create new
        if (!empty($widgetData['alerts']))
        {
            $alertFilter = array();
            $alertFilter['username'] = $username;
            $alertFilter['name']     = $defaultAlertName;
            $alertsTmp = $this->alerts_model->get_only_alerts($alertFilter);

            if (!empty($alertsTmp))
            {
                $alertFound = TRUE;
                $alertData = $alertsTmp[0];
            }
        }

        // alert not found
        // TODO: discuss what shold happen if rule with $defaultRuleName already exist

        // for now assume that we must create rule and alert and then widget if needed
        //create rule
        $sketchComplianceRuleData['name']        = $defaultRuleName."_compliance";
        $sketchComplianceRuleData['username']    = $username;
        $sketchComplianceRuleData['description'] = "Autogenerated condition for the activation '".$activationName . "' of sketch '". $sketchName."'";
        $sketchComplianceRuleData['type']        = 'sketch';

        $sketchComplianceRuleData['sketchActivation'] = array();
        $sketchComplianceRuleData['sketchActivation']['sketchName']     = $sketchName;
        $sketchComplianceRuleData['sketchActivation']['activationName'] = $activationName;
        $sketchComplianceRuleData['sketchActivation']['activationHash'] = $hash;
        $sketchComplianceRuleData['sketchActivation']['sketchCheckType']     = 'compliance';


        $sketchPresenceRuleData = $sketchComplianceRuleData; // make a copy
        $sketchPresenceRuleData['name'] = $defaultRuleName."_presence";
        $sketchPresenceRuleData['sketchActivation']['sketchCheckType']     = 'presence';


        // save rule
        try
        {
           $complianceRuleObj = $this->rules_model->saverule($sketchComplianceRuleData, true, $is_admin);
           $sketchPresenceRuleObj = $this->rules_model->saverule($sketchPresenceRuleData, true, $is_admin);
        }
        catch (Exception $e)
        {
            $message = $e->getMessage();
            log_message('debug', 'Unable to create rule. Error: ' . $message);
            respond_internal_error($message);
            return;
        }

        // usually validation errors
        if ($complianceRuleObj === false ||  $sketchPresenceRuleObj === false  ) {
            $message = $this->rules_model->getErrors();
            log_message('debug', 'Unable to create rule. Error: ' . $message[0]['text']);
            respond_internal_error($message);
            return;
        }


        // insert alert
        $alertData['name']     = $defaultAlertName."_compliance";
        $alertData['username'] = $username;
        $alertData['severity'] = 'high';
        $alertData['ruleid']   = $complianceRuleObj->id;
        $alertData['hostContexts'] = $includes;
        $alertData['paused'] = 0;
        $alertData['pause'] = 0;


        $presenceAlertData  = $alertData; // make a copy
        $presenceAlertData['name']     = $defaultAlertName."_presence";
        $presenceAlertData['ruleid']   =  $sketchPresenceRuleObj->id;
        $presenceAlertData['paused'] = 1;




        // save alert
        try
        {
            $alertObj = $this->alerts_model->savealert($alertData, true, $is_admin);
            $presenceAlertObj = $this->alerts_model->savealert($presenceAlertData, true, $is_admin);

        }
        catch (Exception $e)
        {
            $message = $e->getMessage();
            log_message('debug', 'Unable to create widget. Error: ' . $message);
            respond_internal_error($message);
            return;
        }


        // usually validation errors
        if ($alertObj === false)
        {
            $message = $this->alerts_model->getErrors();
            log_message('debug', 'Unable to create alert. Error: ' . $message[0]['text']);
            respond_internal_error($message);
            return;
        }


        // add alert to widget
        if ($alertFound !== TRUE)
        {
            $widgetData['alerts'][] = $alertObj->id;
            $widgetData['alerts'][] = $presenceAlertObj->id;
        }


        // save /update widget
        try
        {
            $widgetObj = $this->widget_model->savewidget($widgetData, true, $is_admin);
            if (!$widgetObj)
            {
                $message = $this->widget_model->getErrors();
                log_message('debug', 'Unable to create widget. Error: ' . $message[0]['text']);
                respond_internal_error($message);
                return;
            }
            else
            {
                $data = array();
                $data[0] = array('widgetId' => $widgetObj->id, 'text' => $this->lang->line('dashboard_widget_save_success'),  'errorCode' => 200);

                try
                {
                    // convert to array and add widget to current dashboard
                    $widgetArray = (array) $widgetObj;
                    $dashboard = $this->dashboard_model->add_widget($widgetArray);
                }
                catch (Exception $e)
                {
                    $message = $e->getMessage();
                    log_message('debug', 'Unable to add widget to dashboard. Error: ' . $message);
                    respond_internal_error($message);
                }
                respond_ok(json_encode($data));
                return;
            }
        }
        catch (Exception $e)
        {
            $message = $e->getMessage();
            log_message('debug', 'Unable to create widget. Error: ' . $message);
            respond_internal_error($message);
            return;
        }


    }

    /**
     * Remove alert, rule and widget (if it is empty) after removing for DC activation
     *
     * 1) check if RULE exist
     * 2) remove rule (if any alert contain this alert) (deleting rule also delete alerts and update widgets
     * 3) get automatically generated widget
     * 4) if widget from step 4 contain only rule which we deleted - remove widget too
     *
     * @return json
     */
    function removeAlertFromDC_post() {
        $username = $this->session->userdata('username');
        $data = json_decode($this->request->body, true);

        $activationName = $data['activationName'];
        $sketchName     = $data['sketchName'];
        $is_admin       = $this->ion_auth->is_admin();

        $defaultRuleName = $this->_getDefaultRuleName_DC($activationName);

        $complianceRuleName = $defaultRuleName."_compliance";
        $presenceRuleName = $defaultRuleName."_presence";



        $widgetData = array();


        $namesFilter = array($complianceRuleName, $presenceRuleName);

        $this->load->model('rules_model');

        $rulesTmp = $this->rules_model->get_all_rules_having_name($namesFilter,array('username'=>$username));


        // for now assume that if rule is not found - do not do anything
        if (empty($rulesTmp))
        {
            $data[0] = array('text' => $this->lang->line('dashboard_removeAlertFromDC_rule_not_found'),  'errorCode' => 200);
            respond_ok(json_encode($data));
            return;
        }


       foreach ($rulesTmp as  $rulesObj)
       {
            // 2) remove rule, this will also delete all alerts and update widgets asscociated with this rule
            if (!empty($rulesObj->id))
            {
                $this->load->model('rules_model');

                try
                {
                    $this->rules_model->deleterule($rulesObj->id, $username, $is_admin);
                }
                catch (Exception $e)
                {
                    $message = $e->getMessage();
                    log_message('debug', 'Unable to delete rule, error: ' . $message);
                    respond_internal_error($message);
                    return;
                }
            }
        }


// 3) get automatically generated widget

        $widgetFilter = array();
        $widgetFilter['username'] = $username;
        $widgetFilter['name']     = $sketchName;

        $this->load->model('widget_model');

        $widgetTmp  = $this->widget_model->get_all_widgets($widgetFilter);

        if (!empty($widgetTmp))
        {
            $widgetData = (array)$widgetTmp[0];
        }


// 4) Remove empty widget if needed

        if (empty($widgetData['alerts']))
        {
            try
            {
                $this->widget_model->delete_widgets($widgetFilter, $is_admin);
            }
            catch (Exception $e)
            {
                $message = $e->getMessage();
                log_message('debug', 'Unable to delete widget, error: ' . $message);
                respond_internal_error($message);
                return;
            }
        }

        $data[0] = array('text' => $this->lang->line('dashboard_removeAlertFromDC_success'),  'errorCode' => 200);
        respond_ok(json_encode($data));
        return;
    }


    function pausealert_post()
    {
        $data = json_decode($this->request->body, true);


        $dataToSave = array();

        $dataToSave['paused'] = 1;
        $dataToSave['pause']  = ((int)$data['time']) !== -1 ? $data['time'] + time() : -1;

        $alertId = $data['alertid'];


        $username = $this->session->userdata('username');

        $is_admin = $this->ion_auth->is_admin();

        if (!isset($data['alertid']))
        {
            $message = 'Unable to pause alert: Alert id is empty';
            log_message('debug', $message);
            respond_internal_error($message);
            return;
        }

        try
        {
            $filter['id'] = intval($alertId);

            if ($is_admin === false)
            {
                $filter['username'] = $username;
            }

            $this->alerts_model->updateAlert($filter, $dataToSave);
        }
        catch (Exception $e)
        {
            $message = $e->getMessage();
            log_message('debug', 'Unable to pause alert: ' . $message);
            respond_internal_error($message);
            return;
        }

        $result = array();
        $result[0] = array('text' => $this->lang->line('dashboard_alert_paused_success'), 'errorCode' => 200);
        respond_ok(json_encode($result));
        return;
    }

    function resumealert_post()
    {
        $data = json_decode($this->request->body, true);


        $dataToSave = array();

        $dataToSave['paused'] = 0;
        $dataToSave['pause']  = 0;

        $alertId = $data['alertid'];


        $username = $this->session->userdata('username');

        $is_admin = $this->ion_auth->is_admin();

        if (!isset($data['alertid']))
        {
            $message = $e->getMessage();
            log_message('debug', 'Unable to resume alert: Alert id is empty');
            respond_internal_error($message);
            return;
        }

        try
        {
            $filter['id'] = (int)$alertId;

            if ($is_admin === false)
            {
                $filter['username'] = $username;
            }

            $this->alerts_model->updateAlert($filter, $dataToSave);
        }
        catch (Exception $e)
        {
            $message = $e->getMessage();
            log_message('debug', 'Unable to resume alert: ' . $message);
            respond_internal_error($message);
            return;
        }

        $result = array();
        $result[0] = array('text' => $this->lang->line('dashboard_alert_resumed_success'),  'errorCode' => 200);
        respond_ok(json_encode($result));
        return;
    }

    /**
     * Return statistical information for group of alerts by severity
     * result array like
     *  ['high']   = array('failAlerts' => 0, 'totalAlerts' => 0, 'failHosts' => 0, 'totalHosts' => 0);
        ['medium'] = array('failAlerts' => 0, 'totalAlerts' => 0, 'failHosts' => 0, 'totalHosts' => 0);
        ['low']    = array('failAlerts' => 0, 'totalAlerts' => 0, 'failHosts' => 0, 'totalHosts' => 0);
     *
     *
     * @return type
     */
    function getalertstatsbyseverity_post()
    {
        $data = json_decode($this->request->body, true);
        $username = $this->session->userdata('username');
        $cacheKey = $username . implode('_', $data['alertsIds']);

        if ($this->cache->get($cacheKey))
        {
            respond_ok($this->cache->get($cacheKey));
            return true;
        }

        if (empty($data['alertsIds']))
        {
            $message = $this->lang->line('dashboard_alert_list_is_empty');
            log_message('debug', 'Unable get alert info: ' . $message);
            respond_internal_error($message);
            return;
        }

        try
        {
            $alertsIds = array_unique($data['alertsIds']);
            $alertsIds = array_map('intval',$alertsIds);

            $filter = sprintf('((pause != -1 AND pause <= %s) OR pause IS NULL) ',time());

            $alertsList = $this->alerts_model->get_alerts_with_ids($alertsIds, $filter);

            if(empty($alertsList))
            {
                $message = $this->lang->line('dashboard_alert_list_is_empty');
                log_message('debug', 'Unable get alert info: ' . $message);
                respond_internal_error($message);
                return;
            }

            $result = $this->alerts_model->getAlertStats($alertsList);
            // Save into the cache for 5 minutes
            $json = json_encode($result);
            $this->cache->save($cacheKey, $json, 300);
            respond_ok($json);
        }
        catch(Exception $e)
        {
            $message = $e->getMessage();
            log_message('debug', 'Failed retrieving alerts, error: ' . $message);
            respond_internal_error($message);
        }
    }

    /* Utils functions */

    function _getDefaultAlertName_DC($activationName) {
        return $activationName;
    }

    function _getDefaultRuleName_DC($activationName) {
        return $activationName;
    }

 }
?>