<?php
require_once APPPATH . 'libraries/Cf_REST_Controller.php';

class Rulesapi extends Cf_REST_Controller
{

    public $username = "";
    private $roles = null;
    private $availableSketches = array();

    function __construct()
    {
        parent::__construct();

        $this->load->model('rules_model');
        $this->username = $this->session->userdata('username');
    }


    //$rulesIds - can be urlencoded comma separated list
    function rules_get($rulesIds = '')
    {

        $filter = array();
        $filterIds = array();
        //$filter['username'] = $this->session->userdata('username');

        if (!empty($rulesIds))
        {
            $rulesIds = explode(',', urldecode($rulesIds));

            foreach($rulesIds as $item => $id)
            {
                $rulesIds[$item] =  $id;
            }

            $filterIds = array_values($rulesIds);
        }

        try
        {
            $result = $this->rules_model->get_rules_with_ids($filterIds, $filter, array('name' => 'asc'));

            if (empty($result)) {
                $message = $this->lang->line('dashboard_rule_not_exist');
                log_message('debug', 'Failed retrieving rule, error: ' . $message);
                respond_internal_error($message);
                return;
            }

            $json = json_encode($result);
            respond_ok($json);
        }
        catch(Exception $e)
        {
            $message = $e->getMessage();
            log_message('debug', 'Failed retrieving rules, error: ' . $message);
            respond_internal_error($message);
        }
    }

    /**
     * Save rule
     * Allow overwrite only by id
     * same name not allowed  - show error
     *
     *
     * @param <string> $id  - id of the rule. If not provided - assume insert, if provided - assume update
     * @return
     */
    function saverule_post($id = '')
    {

        $data = json_decode($this->request->body, true);

        $data['username'] = $this->session->userdata('username');
        $is_admin = $this->ion_auth->is_admin();


        $overwrite = FALSE;
        if (!empty($id))
        {
            $overwrite = TRUE;
        }

        try
        {
            $obj = $this->rules_model->saverule($data, $overwrite, $is_admin);
        }
        catch(Exception $e)
        {
            respond_internal_error($e->getMessage());
            return;
        }

        if (!$obj) //not able to get inserted rule
        {
            $errors = $this->rules_model->getErrors();
            $data = json_encode($errors);
            respond_internal_error($data);
            return;
        }
        else
        {
            $data = array();
            $data[0] = array('ruleid' => $obj->id, 'text' => $this->lang->line('dashboard_rule_save_success'), 'errorCode' => 200);
            respond_ok(json_encode($data));
            return;
        }
    }

    function validateCustomSql_post(){
        $data = json_decode($this->request->body, true);
        $obj = $this->rules_model->setRestClient($this->getRestClient());
        $obj = $this->rules_model->validateCustomSql($data['sql']);
        respond_ok(json_encode($obj));
    }

    function deleterule_delete($id) {

        $username = $this->session->userdata('username');
        $is_admin = $this->ion_auth->is_admin();

        try
        {
            $result = $this->rules_model->deleterule($id, $username, $is_admin);
            $json = json_encode($result);
            respond_ok($json);
        }
        catch(Exception $e)
        {
            $message = $e->getMessage();
            log_message('debug', 'Unable to delete rule. Error: ' . $message);
            respond_internal_error($message);
        }
    }
 }