<?php
require_once APPPATH . 'modules/dashboard/models/entities/CF_Rule.php';
class rules_model extends Cf_Model
{
    var $collectionName = 'dashboard_rules';
    var $variables_dictionary = 'variables_dictionary';

    var $defaultFilter;


    function __construct()
    {
        parent::__construct();
    }

    function setDefaultFilter($filter)
    {
        $this->defaultFilter = $filter;
    }

    function saverule($data, $overwrite = false, $is_admin = false)
    {
        if (!is_array($data))
        {
            $this->setError('Invalid parameter passed to function');
            return false;
        }


        // set username as default filter
        $filter = array();
        $filter['username'] = $data['username'];

        if (isset($data['id']) && !empty($data['id']))
        {
            $filter['id'] = $data['id'];
        }

        $this->setDefaultFilter($filter);

        if ($this->validate($data, $overwrite, $is_admin))
        {
            $id = '';

            $filter = $this->defaultFilter;
            $filter['name'] = $data['name'];
            $currentItem = $this->get_all_rules($filter);

            if (!empty($currentItem))
            {
                $id = $currentItem[0]->id;
                $reportOwner = $currentItem[0]->getUsername();

                // alert report owner. if this report created by user and get's overwritten by admin, we will try to keep original owner
                // $data['username'] - user who created report, in most cases current logged user
                if (($reportOwner !== $data['username']) && $is_admin === true)
                {
                    $data['username'] = $reportOwner;
                }
            }

            try
            {
                $insertData = $this->_map_for_insert($data);

                if ($id == '')
                { // not exist - insert it will return inserted item

                    $this->db->insert($this->collectionName, $insertData);
                    $id = $this->db->insert_id();
                    log_message('debug', 'Successful save rule.: ' . $id);
                }
                else
                {
                    // if exist - update, and then select inserted item
                    $this->db->where(array(
                        'id' => $id
                    ))->update($this->collectionName, $insertData);
                }
            }
            catch(Exception $e)
            {
                $message = $e->getMessage();
                log_message('debug', 'Unable to save rule. Error: ' . $message);
                throw new Exception($message);
            }

            $offset = '';
            $limit = '';
            $order = array();
            // get object from DB
            $ruleObj = $this->get_all_rules(array(
                'id' => $id
            ) , $offset, $limit, $order, $is_admin);

            if ($ruleObj)
            {
                return $ruleObj[0];
            }
        }
        // in case of validation error we return false and error can be get from getErrors();

        return false;
    }

    function _map_for_insert($params)
    {
        if (!empty($params['policyConditions']))
        {
            $params['policyConditions'] = json_encode($params['policyConditions']);
        }


        if (!empty($params['sketchActivation']))
        {
            $params['sketchActivation'] = json_encode($params['sketchActivation']);
        }

        if(!empty($params['inventoryConditions']))
        {
            $params['inventoryConditions'] = json_encode($params['inventoryConditions']);
        }

        if (!empty($params['softwareUpdateConditions']))
        {
            $params['softwareUpdateConditions'] = json_encode($params['softwareUpdateConditions']);
        }

        if (!empty($params['customConditions']))
        {
            $params['customConditions'] = json_encode($params['customConditions']);
        }

        return $params;
    }

    function validate($data, $overwrite = false, $is_admin = false)
    {
        $valid = true;

        if (!isset($data['username']) || trim($data['username']) == '')
        {
            $valid = false;
            $this->setError('username is empty');
            return false;
        }

        if (!isset($data['name']) || trim($data['name']) == '')
        {
            $valid = false;
            $this->setError($this->lang->line('dashboard_rule_name_empty'));
            return false;
        }

        if (!isset($data['type']) || trim($data['type']) == '')
        {
            $valid = false;
            $this->setError($this->lang->line('dashboard_rule_type_empty'));
            return false;
        }


        switch ($data['type'])
        {
            case 'sketch':
                if (!isset($data['sketchActivation']) || (!isset($data['sketchActivation']['sketchName']) || !isset($data['sketchActivation']['activationName'])) || (trim($data['sketchActivation']['sketchName']) == '') || (trim($data['sketchActivation']['activationName']) == ''))
                {
                    $valid = false;
                    $this->setError($this->lang->line('dashboard_rule_sketch_data_empty'));
                    return false;
                }
                break;

            case 'inventory':
                if (empty($data['inventoryConditions']))
                {
                    $valid = false;
                    $this->setError($this->lang->line('dashboard_rule_inventory_data_empty'));
                    return false;
                }
                break;
            // for software updates - all fields can be empty, so no backend validation is needed
        }


        //alert if user admin or owner of this report, otherwise - do not allow to save
        if ($overwrite === false)
        {
            $item = $this->_nameAlreadyExist($data);

            if (!empty($item))
            {
                $valid = false;
                $this->setError($this->lang->line('dashboard_rule_already_exists') , 422); //422 Unprocessable Entity
            }
        }

        //if ($this->_nameAlreadyExist($data) === TRUE)
        if ($this->_nameAlreadyExist($data) === true && (($is_admin === false) && ($this->_ruleOwner($data) === false)))
        {
            $this->clearErrors();
            $this->setError($this->lang->line('dashboard_rule_already_exists') , 422); //422 Unprocessable Entity
            $valid = false;
        }

        return $valid;
    }


    // TODO: refactor. is_admin is not in use
    function get_all_rules($filter = array() , $offset = '', $limit = '', $order = array() , $is_admin = false)
    {
        if (!empty($limit))
        {
            $this->db->limit($limit, $offset);
        }
        if (!empty($order))
        {
            reset($order);
            $field = key($order);
            $orderValue = current($order);
            $this->db->order_by($field, $orderValue);
        }

        $query = $this->db->where($filter)->get($this->collectionName);
        $ruleResult = $this->getResult($query);
        return $ruleResult;
    }

    function get_rules_with_ids($ids = array() , $extraFilter = array(), $order = array())
    {
        $intIds = array_map("intval", $ids);
        $this->db->where($extraFilter);
        if (!empty($intIds))
        {
            $this->db->where_in('id', $intIds);
        }

        if (!empty($order))
        {
            reset($order);
            $field = key($order);
            $orderValue = current($order);
            $this->db->order_by($field, $orderValue);
        }

        $query = $this->db->get($this->collectionName);
        if (!$query)
        {
            throw new Exception("Unable to retrieve rules due to database query error.");
        }
        $ruleResult = $this->getResult($query);

        return $ruleResult;
    }

    function get_all_rules_having_name($names = array(), $extraFilter = array())
    {
        $this->db->where_in('name', $names);

        if (!empty($extraFilter)) {
            $this->db->where($extraFilter);
        }

        $query = $this->db->get($this->collectionName);
        $ruleResult = $this->getResult($query);
        return $ruleResult;
    }

    function getResult($query)
    {
        $result = array();

        foreach ((array)$query->result_array() as $dbResult)
        {
            $obj = new CF_Rule($dbResult);
            $result[] = $obj;
        }

        return $result;
    }

    function deleterule($id, $username, $is_admin)
    {
        // delete from widgets
        // for now we can not share alerts, so we select only user widgets

        $ruleFilter = array();

        // admin can delete any alert
        if ($is_admin !== TRUE)
        {
            $ruleFilter['username'] = $username;
        }

        //0. check if rule exist
        // database will take care of deleting corresponding alert

        $ruleFilter['id'] = intval($id);

        try
        {
            $return = $this->db->where($ruleFilter)->delete($this->collectionName);
            return $return;
        }
        catch(Exception $e)
        {
            log_message('error', $e->getMessage() . " " . $e->getFile() . " line:" . $e->getLine());
            throw $e;
        }
    }


    function _nameAlreadyExist($data)
    {
        $alreadyExist = FALSE;
        $filter = $this->defaultFilter;
        $filter['name'] = $data['name'];

        $result = $this->get_all_rules($filter);

        if (!empty($result))
        {
            $alreadyExist = TRUE;
        }

        return $alreadyExist;
    }

    function _ruleOwner($data)
    {
        $isOwner = false;
        $filter = $this->defaultFilter;
        $filter['name'] = $data['name'];

        $currentItem = $this->get_all_rules($filter);

        if (!empty($currentItem))
        {
            $owner = $currentItem[0]->getUsername();

            if ($owner === $data['username'])
            {
                $isOwner = true;
            }
        }

        return $isOwner;
    }


   //************************************
    //
    // TODO: This functions should be revieed and replaced in all project if valuable
    // right now this is a copy/paste from save_search model
    //________________________________________________________

    function setError($errorText, $errorCode = 406, $level = 'ERROR')
    {
        $i = count($this->errors);
        $this->errors[$i]['text'] = $errorText;
        $this->errors[$i]['errorCode'] = $errorCode;
    }

    function getErrors($level = 'ERROR')
    {
        return $this->errors;
    }

    function clearErrors()
    {
        $this->errors = array();
    }

    function validateCustomSql($sql)
    {
        $errors = [];
        try {
            $queryResult = $this->rest->post('/query', ["query" => $sql]);
        } catch (Exception $e) {
            $errors[] =  $e->getMessage();
            return $errors;
        }
        $queryResult = json_decode($queryResult, JSON_OBJECT_AS_ARRAY);

	    $isHostKey = false;
        foreach ($queryResult['data'][0]["header"] as $column){
	        if ($column["columnName"] == 'hostkey') {
		        $isHostKey = true;
	        }
        }
        if(!$isHostKey){
	       $errors[] = 'One of the select columns must be hostkey.';
        }
        return $errors;
    }


    // END
    // ________________________________________________________

}