<?php
require_once APPPATH . 'libraries/Cf_REST_Controller.php';

class Dcapi extends Cf_REST_Controller
{

    public $username = "";
    private $roles = null;
    private $availableSketches = array();

    function __construct()
    {
        parent::__construct();

        $this->load->helper('dc_path');
        $this->config->load('designcenter');
        $this->load->model('designcenter/dc_rest_model');

        $this->username = $this->session->userdata('username');
        $this->roles = $this->session->userdata('roles');
    }

    function sketchCategory_get()
    {
        try
        {
            $allSketch = $this->dc_rest_model->getAllAvailableSketches();

            $activations = $this->dc_rest_model->getActivations();

            $data_model = array();


            foreach ($allSketch as $id => $value)
            {

                $explode = explode('::', $id);
                $category = $explode[0]; // first one is the category
                $sketchName = $value['metadata']['name'];
                $deployed = $this->_checkIfDeployed($sketchName, $activations);
                $description = $value['metadata']['description'];
                $tags = $value['metadata']['tags'];
                $insertData = array(
                    'tags' => $tags,
                    'category' => $category,
                    'name' => $sketchName,
                    'description' => $description,
                    'deployed' => $deployed
                );
                if (!key_exists($category, $data_model))
                {
                    $data_model[$category] = array();
                }
                array_push($data_model[$category], $insertData);
            }

            $json = json_encode($data_model);
            respond_ok($json);
        }
        catch(Exception $e)
        {
            $message = $e->getMessage();
            log_message('debug', 'Failed retrieving sketches, error from API: ' . $message);
            respond_internal_error($message);
        }
    }

    function activation_details_get($sketchName)
    {
        $activations = $this->dc_rest_model->getActivations($sketchName);
        respond_ok(json_encode($activations));
    }
    /**
     *
     * @param type $sketchName
     * @return int 0 or 1
     */
    function _checkIfDeployed($sketchName, $activations)
    {
        if (empty($sketchName))
        {
            return 0;
        }

        if (!empty($activations[$sketchName]))
        {
            return 1;
        }
        return 0;
    }

    function sketch_get($sketchId)
    {
        try
        {
            $sketchInfo = $this->dc_rest_model->getSketch($sketchId, true);
            if (!empty($sketchInfo))
            {
                $validationDetails = $this->_getValidationDetails($sketchInfo);
                if (!empty($validationDetails))
                {
                    $sketchInfo['validationDetails'] = $validationDetails;
                }
                $json = json_encode($sketchInfo);
                $this->respond_ok($json);
            }
            else
            {
                $this->respond_not_found("Sketch not found.");
                return;
            }
        }
        catch(Pest_NotFound $e)
        {
            $this->respond_not_found("Sketch not found.");
            return;
        }
        catch(Exception $e)
        {
            $this->respond_internal_error($e->getMessage());
            return;
        }
    }

    function _getValidationDetails($sketchInfo)
    {
        $listOfValidation = array();
        $validationInfo = array();


        foreach ($sketchInfo["api"] as $api)
        {


            foreach ($api as $inputs)
            {
                $type = $inputs['type'];
                $validation = !empty($inputs['validation']) ? $inputs['validation'] : false;
                if ($type == 'array' && $validation)
                {
                    $listOfValidation[] = $validation;
                }
            }
        }

        $commaSeparatedList = implode(',', $listOfValidation);
        if (!empty($commaSeparatedList))
        {
            // fire up rest request to get the details
            $validationResultJson = $this->dc_rest_model->getValidations(null, $commaSeparatedList);
            $validationResult = json_decode($validationResultJson, true);

            $validationInfo = $validationResult['data'];
        }

        return $validationInfo;
    }

    function removeEmptyValues($params)
    {


        foreach ((array)$params as $index => $p)
        {
            if (is_array($p))
            {
                if (empty($p))
                {
                    unset($params[$index]);
                }
            }
            elseif (strlen($p) == 0)
            {
                // this will not remove 0 or false value
                unset($params[$index]);
            }
        }
        return $params;
    }

    function activate_post()
    {

        $postData = json_decode($this->request->body, true);

        $sketchName = $postData['sketchId'];
        $environment = $postData['environment']['includes'];

        $params = $postData['params'];
        $activationDescription = $params['CF_MP_INTERNAL_SKETCH_activationName'];
        $bundleName = $params['CF_MP_ENTRY_POINT'];

        unset($params['CF_MP_INTERNAL_SKETCH_activationName']);
        $envName = md5(serialize($environment));
        $uid = md5(uniqid($activationDescription, true));
        $paramName = $uid;
        $metaData = array('environmentLabel' => $postData['environment']['name']);

        try
        {

            $install = $this->dc_rest_model->install($sketchName);

            $params = $this->removeEmptyValues($params); // remove empty elements

            $defParams = $this->dc_rest_model->defineParams($sketchName, $paramName, $params);

            $defEnv = $this->dc_rest_model->defineEnvironment($envName, $environment);

            $activate = $this->dc_rest_model->activate($sketchName, $envName, $bundleName, $paramName, $activationDescription, $metaData);

            $this->respond_ok($activate);
        }
        catch(Exception $e)
        {
            log_message('debug', $e->getMessage());
            $this->respond_internal_error($e->getMessage());
        }

        $this->respond_ok();
    }

    function commit_post()
    {

        $userData = $this->ion_auth->get_user($this->session->userdata('username'));
        $postData = json_decode($this->request->body, true);
        $msg = $postData['commitMessage'];
        $userEmail = !empty($userData['email']) ? $userData['email'] : '';
        try
        {
            $this->dc_rest_model->commit($msg, $userEmail);
        }
        catch(Exception $e)
        {
            $message = $e->getMessage();
            $this->respond_internal_error($message);
        }

        $this->respond_ok();
    }

    function validate_post()
    {

        $postData = json_decode(trim(utf8_encode($this->request->body)) , true);

        $validationType = $postData['validationType'];
        $validationData = $postData['validationData'];

        try
        {
            $response = $this->dc_rest_model->validate($validationType, $validationData);
            $status = $response['api_ok']['success'];
            $error = $response['api_ok']['errors'];
            $respond = array(
                'status' => $status,
                'error' => $error
            );
        }
        catch(Exception $e)
        {
            // we have error here
            $status = false;
            $error = $e->getMessage();
            $respond = array(
                'status' => $status,
                'error' => $error
            );
        }

        $json = json_encode($respond);
        $this->respond_ok($json);
    }
    // reset and pull the latest changes
    function git_reset_post()
    {
        try
        {

            $this->dc_rest_model->reset_and_pull();
        }
        catch(Exception $e)
        {
            $message = $e->getMessage();
            $this->respond_internal_error($message);
        }
        $this->respond_ok();
    }

    function deactivate_post($id, $sketchName)
    {
        $id = html_entity_decode(urldecode($id),ENT_QUOTES,'UTF-8');
        $sketchName = html_entity_decode(urldecode($sketchName),ENT_QUOTES,'UTF-8');
        try
        {
            $apiResponse = $this->dc_rest_model->deactivate($id,$sketchName);
            $this->respond_ok();
        }
        catch(Exception $e)
        {
            $message = $e->getMessage();
            $this->respond_internal_error($message);
        }

        $this->respond_ok();
    }

    function runagent_get($hostkey = '')
    {
        $this->config->set_item('compress_output', false);

        $user_role = $this->session->userdata('roles');
        // check if the user has cf_remoteagent role
        if (in_array('cf_remoteagent', $user_role) === false)
        {

            $this->respond_internal_error('You are not authorized to run cf-runagent. Please contact your administrator.');
        }

        if (!empty($hostkey))
        {
            $this->load->model('host_model');
            $ip = $this->host_model->getHostIp($this->username, $hostkey);
            if ($ip)
            {

                // check for sudo access if not show error
                $returnVar = 0;
                $command = 'sudo -v';
                $out = array();
                $sudoCheck = exec ( $command , $out, $returnVar );
                if ($returnVar)
                {
                    // no sudo access detected
                    $error = lang('cf-runagent-access-error');
                    $errorMessage = '<pre>' . $error . '</pre>';
                    show_error($errorMessage);
                    exit;
                }

                flush();
                $hailIp = escapeshellarg($ip);
                $command = "sudo -n /var/cfengine/bin/cf-runagent --hail $hailIp -o '-v'";

                $descriptorspec = array(
                    1 => array(
                        'pipe',
                        'w'
                    ) ,
                    2 => array(
                        'pipe',
                        'w'
                    ) ,
                );
                $pipes = array();
                $resource = proc_open($command, $descriptorspec, $pipes);


                while ($stdout = fgets($pipes[1]))
                {
                    print '<pre>' . $stdout . '</pre>';
                    flush();
                }

                $stderr = stream_get_contents($pipes[2]);


                foreach ($pipes as $pipe)
                {
                    fclose($pipe);
                }

                $status = trim(proc_close($resource));

                if ($status && !empty($stderr))
                {
                    log_message('error', $stderr);
                    $this->respond_internal_error($stderr);
                }
            }
        }
        else
        {
            $this->respond_ok('No hostkey detected');
        }
    }

    function promise_notkept_log_get($key, $sketch, $identifier)
    {

        $identifier = urldecode($identifier);
        $this->load->model('advancedreports_model');

        $sql = sprintf("select p.changetimestamp,p.promisehandle,unnest(p.logmessages) as messages
               from promiseexecutions p
               where p.hostkey = '%s'
               and p.promiseoutcome = 'NOTKEPT'
               and '{\"%s\",\"%s\"}' <@ p.promisees
               ORDER BY p.changetimestamp ASC", $key, $sketch, $identifier);


        $sqlNotKeptResult = $this->advancedreports_model->runQuery($this->username, $sql, $sortColumn = '', $sortDescending = '', $skip = 0, $limit = - 1, $includes = array() , $excludes = array());

        $this->respond_ok(json_encode($sqlNotKeptResult));
    }

    function hostlist_post()
    {
        $this->load->model('advancedreports_model');
        $postData = json_decode(trim(utf8_encode($this->request->body)) , true);
        $includeregex = !empty($postData['includes']) ? $postData['includes'] : array();
        $osList = !empty($postData['dependentOs'][0]) && is_array($postData['dependentOs'][0]) ? array_keys($postData['dependentOs'][0]) : array();
        if (!empty($osList))
        {
            $supportedOs = implode("','", $osList);
            $commaSeparatedList = "'" . $supportedOs . "'";
            $sql = sprintf('SELECT h.hostkey,h.hostname,(select count(1) from contexts c where contextname in (%s) and c.hostkey = h.hostkey) as hasOS from hosts h', $commaSeparatedList);
            $sqlResult = $this->advancedreports_model->runQuery($this->username, $sql, $sortColumn = '', $sortDescending = '', $skip = 0, $limit = 50000, $includeregex, array());
            $this->respond_ok(json_encode($sqlResult));
            return;
        }
        else
        {
            $message = "The supported OS list is either empty or has the wrong format specified in the sketch metadata.";
            log_message('error', $message);
            $this->respond_internal_error($message);
        }
    }

    function validation_get($key = null)
    {
        try
        {
            $apiResponse = $this->dc_rest_model->getValidations($key);
            $this->respond_ok($apiResponse);
            return;
        }
        catch(Exception $e)
        {
            $message = $e->getMessage();
            $this->respond_internal_error($message);
            return;
        }
    }
}
?>