<?php

/**
 * Class LdapSettingsTest
 * @covers \models\SettingsModel
 */
class LdapSettingsTest extends PHPUnit_Framework_TestCase
{

    private $settingsModel;

    public function __construct()
    {
        $this->settingsModel = new \models\SettingsModel(\tests\App::getInstance()->getContainer());
        parent::__construct();
    }


    public function testValidateOk()
    {
        $data = ['base_dn' => 'base_dn', 'timeout' => 5];
        $this->assertTrue($this->settingsModel->validate($data), 'Validation error');
    }

    public function testValidateWrongBool()
    {
        $data = ['base_dn' => 'base_dn', 'use_ssl' => 'no'];
        $this->expectException(\Exception::class);
        $this->settingsModel->validate($data);
    }

    public function testValidateWrongInteger()
    {
        $data = ['base_dn' => 'base_dn', 'port' => 'ff22222'];
        $this->expectException(\Exception::class);
        $this->settingsModel->validate($data);
    }


    public function testUpdate()
    {
        $dataToUpdate = ['base_dn' => 'base_dn', 'port' => mt_rand(1, 9999)];
        $currentSettings = $this->settingsModel->getAll();
        $this->assertTrue($this->settingsModel->update($dataToUpdate), 'Error with backup creating');
        $this->assertNotEquals($currentSettings['port'], $dataToUpdate['port'], 'Error while updating');
    }

    public function testMakeBackup()
    {
        $this->assertTrue($this->settingsModel->makeBackup(), 'Error with backup creating');
    }

}