/*jslint node: true */
/*jslint browser: true*/
/*jslint nomen: true*/
/*jslint plusplus: true */
/*global dashboardApp, angular, $*/

'use strict';

dashboardApp.factory('inventoryWidgetHelper',
    ['$q', 'inventoryService', '$window',
        function ($q, inventoryService, $window) {
            var helper = {};

            helper.variable = '';
            helper.reportURL =   '';

            helper.pieChartConfig = {
                options: {
                    chart: {
                        type: 'pie',
                        backgroundColor: 'rgba(255, 255, 255, 255)',
                        height: '194',
                        events:  {
                            click: function () {
                                var reportURL = MP_SITE_URL + "advancedreports#/inventory?columns=Host name," + this.options.chart.attribute;
                                $window.location.href = reportURL;
                            }
                        },
                        attribute: ''
                    },

                    exporting: {
                        enabled: false
                    },
                    plotOptions: {
                        pie: {
                            allowPointSelect: true,
                            cursor: 'pointer',
                            center: [70, 65],
                            dataLabels: {
                                enabled: false
                            },
                            showInLegend: true
                        }
                    },
                    tooltip: {
                        formatter: function () {
                            return this.key + ': ' + this.y;
                        }
                    },
                    legend: {
                        layout: 'vertical',
                        backgroundColor: '#FFFFFF',
                        floating: true,
                        align: 'left',
                        width: 230,
                        verticalAlign: 'top',
                        navigation: {
                            arrowSize: 7,
                            animation: true,
                            style: {
                                fontWeight: 'bold',
                                color: '#333',
                                fontSize: '15px'
                            }
                        },
                        x: 200,
                        y: 0,
                        title: {
                            text: '',
                        },
                        labelFormatter: function () {
                            return helper.getLabel(this)
                        },
                        itemWidth: 190,

                        itemStyle: {
                            color: '#333333',
                            fontSize: '14px',
                            textOverflow: 'ellipsis',
                            width: 190
                        },
                        useHTML: true

                    },

                },
                series: [{
                    name: helper.variable,
                    colorByPoint: true,
                    data: []
                }],
                title: {
                    text: ''
                },
                subtitle: {
                    text: ''
                },
                credits: {
                    enabled: false
                },
                loading: false
            };

            helper.barChartConfig = {
                options: {
                    chart: {
                        type: 'bar',
                        backgroundColor: 'rgba(255, 255, 255, 255)',
                        height: '194',
                        events: {
                            click: function () {
                                var reportURL = MP_SITE_URL + "advancedreports#/inventory?columns=Host name," + this.options.chart.attribute;
                                $window.location.href = reportURL;
                            }
                        },
                        attribute: ''
                    },
                    colors: ['#0071a6'],
                    exporting: {
                        enabled: false
                    },
                    plotOptions: {
                        column: {
                            allowPointSelect: false,
                            cursor: 'pointer'
                        },
                        series: {
                            borderWidth: 0,
                            dataLabels: {
                                enabled: true,
                                format: '{point.y}'
                            }
                        }
                    },
                    tooltip: {
                        formatter: function () {
                            return this.x + ': ' + this.y;
                        }
                    },
                    legend: {
                        enabled: false,
                    }
                },
                series: [{
                    data: []
                }],
                subtitle: {
                    text: ''
                },
                title: {
                    text: '',
                    align: 'left'
                },
                xAxis: {
                    categories: [],
                    title: {
                        text: null
                    },
                    allowDecimals: false,
                    labels: {
                        formatter: function() {
                            return this.value.length > 16 ? (this.value.substring(0,16) + "...") : this.value;
                        }
                    }
                },
                yAxis: {
                    title: {
                        text: ''
                    },
                    labels: {
                        overflow: 'justify'
                    }
                },
                credits: {
                    enabled: false
                },
                loading: false
            };


            helper.getInventoryStats = function (data) {

                var promise = inventoryService.getInventoryStats(data).then(
                    function (result) {
                        return result;
                    },
                    function (error) {
                        // error
                    }
                );

                return $q.all(promise)
            };


            helper.getLabel = function (data) {
                var name = data.name;
                if (name.length > 15) {
                    name = name.substr(0, 15) + '..';
                }
                return '<div><span style="float: left;">' + name + ': </span>' + '<span style="float: right">' + data.y + '</span></div>';
            };


            helper.pieConvertArray = function (arr) {
                var returnData = {};
                returnData.array = [];
                angular.forEach(arr.data, function (value, key) {
                    returnData.array.push({name: key, y: value});
                });
                return returnData;
            };

            helper.barConvertArray = function (arr) {
                var returnData = {};
                returnData.array = [];
                returnData.axis = [];
                returnData.total = 0;
                angular.forEach(arr.data, function (value, key) {
                    returnData.array.push(value);
                    returnData.total += value;
                    returnData.axis.push(key);
                });
                return returnData;
            };

            helper.getChartData = function (type, data) {
                var config = {};

                switch (type){
                    case 'bar':
                        helper.prepareBarChartData(data);
                        config = $.extend(true,{},helper.barChartConfig);
                        break;
                    case 'pie':
                        helper.preparePieChartData(data);
                        config = $.extend(true,{},helper.pieChartConfig);
                        break;
                }
                return config;
            };

            helper.prepareBarChartData = function (data) {
                var barConvertedData = helper.barConvertArray(data);
                helper.barChartConfig.series[0] = {
                    data: barConvertedData.array
                };
                helper.barChartConfig.xAxis.categories = barConvertedData.axis;
                helper.barChartConfig.yAxis.title.text = 'Hosts | Total: ' + barConvertedData.total;
                if(barConvertedData.array.length > 4){
                    helper.barChartConfig.options.chart.height = 50 * barConvertedData.array.length;
                }else {
                    helper.barChartConfig.options.chart.height = 194;
                }
                helper.barChartConfig.options.chart.attribute =  helper.variable;
            }

            helper.preparePieChartData = function (data) {
                var pieConvertedData = helper.pieConvertArray(data);
                helper.pieChartConfig.series[0].data = pieConvertedData.array;
                helper.pieChartConfig.options.legend.title.text = helper.getLegendTitle();
                helper.pieChartConfig.options.chart.attribute = helper.variable;
            }

            helper.getLegendTitle = function () {
              return  '<div style="width: 225px;"><span style="float: left;">' + helper.variable + ': </span><span style="float: right">Hosts</span></div>';
            }

            helper.setVariable = function (variable) {
              helper.variable = variable;
            };



            return helper;
        }]);
