/**
 * environment Directive
 *
 * Use of Class.js
 *
 */
var arrayInputType = BaseController.extend({
    /**
     * @override
     */

    selectClassOne: null,
    operator: null,
    selectClassTwo: null,
    maxRecordPerPage: 10,
    init: function($scope, $elm, $attrs, formController, sketchService, $filter, $q) {
        var self = this;
        this._element = $elm;
        this.filter = $filter;
        this._attrs = $attrs;
        this.formController = formController;
        this.$sketchService = sketchService;
        this.$q = $q;
        this._super($scope);
        this._setValidation();
        this._populateKeysForTypeAhead($scope.inputs);

    },
    /**
     * @override
     */
    defineListeners: function() {
        var self = this;
        // check for validation
    },

    defineScope: function() {
        var self = this;
        var $scope = this.$scope;
        $scope.typeahead = {};
        $scope.dcServerValidationMessage = {};
        $scope.editValidationMessage = {};
        $scope.editDisplay = {};
        $scope.editItem = {};
        $scope.forms = $scope.forms || {};


        $scope.addNewItems = this.addNewItems.bind(this);
        $scope.removeNewItem = this.removeNewItem.bind(this);
        $scope.edit = this.edit.bind(this);
        $scope.changeEditView = this.changeEditView.bind(this);

    },

    edit: function($index, oldKey, oldValue) {

        var self = this;


        newValue = this.$scope.editItem[$index].value;
        newKey = this.$scope.editItem[$index].key;

        var updateFailed = function(data) {
                self.$scope.editValidationMessage[oldKey] = data.error;
            };
        var updateSuccess = function() {
                if (oldKey != newKey) {
                    self.removeNewItem(oldKey, true);
                }
                self.$scope.editValidationMessage = {};
                self.$scope.editDisplay[$index] = 0;
            };

        this.addNewItems(newKey, newValue, true).then(updateSuccess, updateFailed);


    },

    changeEditView: function($index,flag,key,value) {
        this.$scope.editDisplay[$index] = flag;
        this.$scope.editItem[$index]  = {};
        this.$scope.editItem[$index].key = key;
        this.$scope.editItem[$index].value = value;

    },

    removeNewItem: function(key, noprompt) {
        var self = this;
        if (!noprompt) {
            var c = confirm("Are you sure you want to delete this?");
            if (!c) return;
        }

        try {
            delete self.$scope.forms[key];
        } catch (e) {
            self.$scope.forms[key] = undefined;
        }

        var isEmpty = angular.equals({}, self.$scope.forms);
        if (isEmpty) {
            self.$scope.forms = null;
        }

        this._setValidation();
    },

    _populateKeysForTypeAhead: function(details) {
        var self = this;
        var key = details.name;
        var validationKey = details['validation'];

        var populateKey = function(validationDataResponse) {
                var validationData = validationDataResponse.data[validationKey];
                var arrayMapStrict = validationData.array_map_strict || [];
                var arrayMapPermissive = validationData.array_map_permissive || [];
                var options = angular.extend({}, arrayMapStrict, arrayMapPermissive);
                self.$scope.typeahead[key] = [];
                angular.forEach(options, function(value, key) {
                    this.push(key);
                }, self.$scope.typeahead[key]);
            };

        this.$sketchService.getValidationDetails(validationKey).success(populateKey);


    },
    _setValidation: function() {
        var details = this.$scope.inputs;
        var data = this.$scope.forms || {};
        var hasDefaults = details.hasOwnProperty('default');
        var isEmpty = angular.equals({}, data);
        if (!hasDefaults && isEmpty) {
            this.formController.$setValidity(details.name, false);
        } else {
            this.formController.$setValidity(details.name, true);
        }

    },


    addNewItems: function(k, v, noGlobalError) {
        var self = this;
        var deferred = self.$q.defer();
        if (!k || !v) {
            var data = {};
            data.error = 'Either key or value is empty';
            deferred.reject(data);
            return deferred.promise;
        }

        var input = self.$scope.inputs.name;
        var validation = self.$scope.inputs.validation;
        self.$scope.dcServerValidationMessage[input] = null;


        function addItemInArray() {

            if (!angular.isObject(self.$scope.forms)) {
                self.$scope.forms = {};
            }

            self.$scope.forms[k] = v;

            if (self.$scope.typeahead[input] && self.$scope.typeahead[input].indexOf(k) == -1) {
                self.$scope.typeahead[input].push(k);
            }
            self.$scope.itemValue = '';
            self.$scope.itemKey = '';
            deferred.resolve();
            self._setValidation();

        }


        // do some validation
        var validateObj = {};
        validateObj[k] = v;
        if (validation) {
            this.$sketchService.validate(validation, validateObj).success(

            function(data) {
                if (data.status) {
                    addItemInArray();
                } else {
                    // validation error
                    if (!noGlobalError) self.$scope.dcServerValidationMessage[input] = data.error;
                    deferred.reject(data);
                }
            });
        } else {
            addItemInArray();
        }

        return deferred.promise;

    }


});