/**
 * Sketch category  Directive
 *
 * Use of Class.js
 *
 */
var SketchCategoryDirective = BaseController.extend({
    selectedFilterMenu: 'all',
    /**
     * @override
     */
    init: function($scope, $elm, $attrs, sketchService, $filter) {
        var self = this;
        this._element = $elm;
        this._attrs = $attrs;
        this.$filter = $filter;
        this._super($scope);
        this.sketchService = sketchService;
        this.$scope.errors = false;
        this.$scope.isCategoryLoading = true;
        this.sketchService.loadSketchCategories().then(

        function(result) {
            if (result.data.length === 0) {
                self.$scope.errors = 'No sketches found.';
            } else {
                self.$scope.sketches = result.data;
                self.$scope.filteredSketchList = result.data;
                self.populateTagList(self.$scope.sketches);

            }
              this.$scope.isCategoryLoading = false;
        },

        function(error) {
            self.$scope.errors = 'Error getting sketches. ' + error.data;
            this.$scope.isCategoryLoading = false;
        });

    },
    defineScope: function() {
        var self = this;
        $scope = this.$scope;
        $scope.showAll = true;
        $scope.filterCriteria = {
            'name': null,
            'category': [],
            'tags': [],
            'deployed': null
        };
        $scope.categoryList = [];
        $scope.tagList = [];
        $scope.searchTerm = null;
        $scope.filteredSketchList = {};
        $scope.setActiveClassForSketchFilters = this.setActiveClassForSketchFilters.bind(this);
        $scope.searchSketch = this.searchSketch.bind(this);
        $scope.selectCategory = this.selectCategory.bind(this);
        $scope.selectTags = this.selectTags.bind(this);
        $scope.isSelectedCategory = this.isSelectedCategory.bind(this);
        $scope.isSelectedTags = this.isSelectedTags.bind(this);
        $scope.expandAll = false;
        $scope.selectedFilterMenu = 'all';


    },

    defineListeners: function() {
        var self = this;
        $scope = this.$scope;
        /**
         * For filtering the sketch
         * Not made as filter because of angular throwing digest error.
         *
         */
        $scope.$watch('filterCriteria', function(oldValue, newValue) {
            if (angular.equals(oldValue, newValue)) {
                return;
            }

            var filters = $scope.filterCriteria;
            var $filter = self.$filter;

            function _filterCategories(searchCollection) {


                if (filters.category && filters.category.length > 0) {
                    // remove category if not in filter from the collection
                    angular.forEach(searchCollection, function(sketchInfo, key) {
                        if (_.indexOf(filters.category, key) === -1) {
                            delete searchCollection[key];
                        }
                    });
                }
                return searchCollection;
            }

            function _filterDeployed(searchCollection) {
                if (filters.deployed === true) {
                    angular.forEach(searchCollection, function(sketchCollection, category) {
                        var result = $filter('filter')(sketchCollection, {
                            'deployed': 1
                        });
                        if (result.length <= 0) {
                            delete searchCollection[category];
                        } else {
                            searchCollection[category] = result;
                        }

                    });
                }
                return searchCollection;
            }


            function _filterName(searchCollection) {

                if (filters.name && filters.name.length > 0) {

                    angular.forEach(searchCollection, function(sketchCollection, category) {
                        var result = [];

                        angular.forEach(filters.name, function(name, index) {
                            var nameRegex = new RegExp(name, "i");
                            var list = $filter('filter')(sketchCollection, function(sketch) {
                                if (sketch.name.search(nameRegex) !== -1 || sketch.description.search(nameRegex) !== -1) {
                                    return true;
                                }
                                return false;

                            });
                            if (list.length) {
                                // Insert the list in the result array
                                angular.forEach(list, function(value, key) {

                                    var found = _.findWhere(result, value);
                                    if (found === undefined) {

                                        result.push(value);
                                    }
                                });
                            }
                        });
                        if (result.length) {
                            searchCollection[category] = result;
                        } else {
                            delete searchCollection[category];
                        }
                    });
                }

                return searchCollection;
            }

            function _filterTags(collection) {
                if (filters.tags && filters.tags.length > 0) {
                    angular.forEach(collection, function(sketchCollection, category) {
                        var result = [];
                        angular.forEach(sketchCollection, function(sketchInfo, key) {
                            var tagsFound = _.intersection(sketchInfo.tags, filters.tags);
                            if (tagsFound.length > 0) {
                                result.push(sketchInfo);
                            }
                        });
                        if (result.length > 0) {
                            collection[category] = result;
                        } else {
                            delete collection[category];
                        }

                    });
                }
                return collection;
            }

            var copyCollection = angular.copy($scope.sketches, copyCollection);
            $scope.filteredSketchList = _filterName(_filterDeployed(_filterTags(_filterCategories(copyCollection))));
            if (angular.equals({},$scope.filteredSketchList)) {
                $scope.filteredSketchList = null;
            }


        }, true);

        this._super();
    },


    populateTagList: function(sketches) {

        var tagHash = {};

        angular.forEach(sketches, function(sketchInfo, category) {

            angular.forEach(sketchInfo, function(sketch, index) {
                angular.forEach(sketch.tags, function(tag, index) {
                    counter = (!tagHash[tag]) ? 1 : tagHash[tag].counter + 1;
                    tagHash[tag] = {
                        'name': tag,
                        'counter': counter
                    };
                });
            });
        });

        this.$scope.tagList = tagHash;

    },

    setActiveClassForSketchFilters: function(menuItem) {
        if (this.selectedFilterMenu === menuItem) {
            return 'active';
        }

    },

    clean: function(arrayCollection, deleteValue) {
        for (var i = 0; i < arrayCollection.length; i++) {
            if (arrayCollection[i] == deleteValue) {
                arrayCollection.splice(i, 1);
                i--;
            }
        }
        return arrayCollection;
    },

    parseSearchTerm: function(searchTerm) {
        var searchModel = {
            'name': []
        };

        var categories = [];

        vars = searchTerm.split(/\s+/g);

        for (var i = 0; i < vars.length; i++) {
            var pair = vars[i].split(':');
            if (pair.length == 2) {
                var key = pair[0].trim();
                var value = pair[1].trim();
                if (key == 'category' || key == 'tags') {
                    var v = this.clean(value.split(','), '');
                    searchModel[key] = v;
                } else {
                    searchModel[key] = value;
                }
            } else {

                searchModel['name'].push(pair[0].trim());
            }

        }
        return searchModel;
    },

    searchSketch: function(searchTerm) {
        var searchModel = this.parseSearchTerm(searchTerm);
        this.$scope.filterCriteria = angular.extend(this.$scope.filterCriteria, searchModel);
    },

    selectCategory: function(cat) {

        var index = this.$scope.filterCriteria['category'].indexOf(cat);
        if (index !== -1) {
            this.$scope.filterCriteria['category'].splice(index, 1);
        } else {
            this.$scope.filterCriteria['category'].push(cat);
        }

    },
    selectTags: function(tag) {
        var index = this.$scope.filterCriteria['tags'].indexOf(tag);
        if (index !== -1) {
            this.$scope.filterCriteria['tags'].splice(index, 1);
        } else {
            this.$scope.filterCriteria['tags'].push(tag);
        }

    },
    isSelectedCategory: function(cat) {
        var index = this.$scope.filterCriteria['category'].indexOf(cat);
        return (index === -1) ? false : true;
    },
    isSelectedTags: function(tag) {
        var index = this.$scope.filterCriteria['tags'].indexOf(tag);
        return (index === -1) ? false : true;
    }


});